/*
 * This file is made available under the terms of the LGPL licence.
 * This licence can be retreived from http://www.gnu.org/copyleft/lesser.html.
 * The source remains the property of the YAWL Group.  The YAWL Group is a collaboration of
 * individuals and organisations who are commited to improving workflow technology.
 *
 */

import java.util.HashMap;

import service.*;
import org.jdom.Element;

import yawlservice.*;

public class DeclareService extends AbstractYawlService {

	private static final int SERVER_PORT = 8188;
	public static final int CLIENT_PORT = 8189;

	private DeclareServiceServer server;
	private HashMap<String, ExternalCase> launchedCases;

	/**
   *
   */
	public DeclareService() {
		// we log everything to file
		super(
				"c:\\Documents and Settings\\MPESIC\\Desktop\\declareServiceLog1.txt");
		launchedCases = new HashMap<String, ExternalCase>();
		server = new DeclareServiceServer(SERVER_PORT) {

			public void received(YAWLMessage msg) {
				DeclareService.this.received(msg);
			}

			public String response(YAWLMessage msg) {
				return DeclareService.this.response(msg);
			}

		};
		server.start();
	}

	/**
	 * 
	 * @param external
	 *            ExternalWorkItem
	 */
	protected void doMyService(ExternalWorkItem external) {
		sendToDeclare(new StartWorkItemMessage(external));
	}

	private void sendToDeclare(YAWLMessage msg) {
		DeclareServiceClient client = getDeclareServiceClient();
		if (client != null) {
			if (client.send(msg)) {
				getLog().log("SENDING TO DECLARE \n " + msg.toString());
			} else {
				getLog().log("FAILED TO SEND TO DECLARE \n " + msg.toString());
			}
		} else {
			getLog().log("FAILED TO CONNECT TO DECLARE \n " + msg.toString());
		}
	}

	private DeclareServiceClient getDeclareServiceClient() {
		try {
			return new DeclareServiceClient(CLIENT_PORT);
		} catch (Exception ex) {
			return null;
		}
	}

	/*
	 * private boolean send(YAWLMessage msg){
	 * 
	 * return false; }
	 */

	private void received(YAWLMessage msg) {
		if (msg instanceof CompleteWorkItemMessage) { // it's a Complete
														// WorkItem Request
			ExternalWorkItem wi = ((WorkItemMessage) msg).getWorkItem();
			getLog().logEvent("CHECKING IN", wi);
			checkInWorkItem(wi);
		} else {
			// add here for starting a case
		}
	}

	private String response(YAWLMessage msg) {
		if (msg instanceof LaunchCaseMessage) { // it's a LaunchCase Request
			ExternalCase extCase = ((LaunchCaseMessage) msg).getCase();
			String caseId = startCase(extCase);
			launchedCases.put(caseId, extCase);
			return caseId;
		}
		return "";
	}

	private String startCase(ExternalCase extCase) {
		String caseId = launchExternalCase(extCase);
		if (successful(caseId)) {
			extCase.setCaseID(caseId);
			getLog().log("STARTED DECLARE CASE with id: " + caseId);
			return caseId;
		}
		return "";
	}

	protected void completedCase(String caseId, Element data) {
		ExternalCase extCase = launchedCases.get(caseId);
		if (extCase != null) {
			sendToDeclare(new CompleteCaseMessage(extCase));
		}
	}
}
