package yawlservice;

import java.util.*;
import javax.xml.parsers.*;

import org.apache.xerces.dom.*;
import org.w3c.dom.*;


public class ExternalCase {

  private static final String XML_MAIN = "externalcase";
  private static final String DATA = "data";
  private static final String CASE_ID = "case";
  private static final String FILE_PATH = "path";
  private static final String FILE_NAME = "file";
  private static final String SPECIFICATION_ID = "specification";
  private static final String DECOMPOSITION_ID = "decomposition";

  private String _caseID;
  private String _fileName;
  private String _filePath;
  private String _specificationID;
  private String _decompositionID;
  private List<CaseDataElement> _data;

  public ExternalCase() {
    _caseID = "null";
    _fileName = "";
    _filePath = "";
    _specificationID = "";
    _decompositionID = "";
    _data = new ArrayList<CaseDataElement>();
  }

  public ExternalCase(String filePath, String fileName, String specificationID, String decompositionID) {
    this();
    this.setFilePath(filePath);
    this.setFileName(fileName);
    this.setSpecificationID(specificationID);
    this.setDecomopositionID(decompositionID);
  }

  public ExternalCase(ExternalCase ec) {
    this();
    if (ec != null){
      this.setCaseID(ec.getCaseID());
      this.setFilePath(ec.getFilePath());
      this.setFileName(ec.getFileName());
      this.setSpecificationID(ec.getStecificationID());
      this.setDecomopositionID(ec.getDecomopositionID());
    }
  }

  public int hashCode(){
    return _caseID.hashCode();
  }
  
  public boolean equals(Object o){
	  if (o instanceof ExternalCase){
		  return ((ExternalCase) o)._caseID.equals(_caseID);
	  }
	  return false; 
  }

  public Object clone(){
    return new ExternalCase(this);
  }

  public void addData(CaseDataElement data) {
    if (data.getName() != null) {
      _data.add(data);
    }
  }

  public Element toElement() {
    Element root = null;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      DOMImplementation domImpl = builder.getDOMImplementation();
      Document document = domImpl.createDocument(null, "available", null);
      root = document.createElement(XML_MAIN);

      Element caseID = document.createElement(CASE_ID);
      caseID.appendChild(document.createTextNode(_caseID));
      root.appendChild(caseID);

      Element path = document.createElement(FILE_PATH);
      path.appendChild(document.createTextNode(_filePath));
      root.appendChild(path);

      Element file = document.createElement(FILE_NAME);
      file.appendChild(document.createTextNode(_fileName));
      root.appendChild(file);


      Element specID = document.createElement(SPECIFICATION_ID);
      specID.appendChild(document.createTextNode(_specificationID));
      root.appendChild(specID);

      Element decID = document.createElement(DECOMPOSITION_ID);
      decID.appendChild(document.createTextNode(_decompositionID));
      root.appendChild(decID);

      Element data = this.dataToElement(document);
      root.appendChild(data);
    }
    catch (ParserConfigurationException e) {
      e.printStackTrace();
    }
     return root;
  }

  public void fromElement(Element element) {
    if (element != null) {
      this._caseID = XMLParser.getSimpleElementText(element, CASE_ID);
      this._filePath = XMLParser.getSimpleElementText(element, FILE_PATH);
      this._fileName = XMLParser.getSimpleElementText(element, FILE_NAME);
      this._specificationID = XMLParser.getSimpleElementText(element,
          SPECIFICATION_ID);
      this._decompositionID = XMLParser.getSimpleElementText(element,
          DECOMPOSITION_ID);
      this.dataFromElement(element);
    }
  }

  public void dataFromString(String externalWorkItemXML){
    try {
      Node node = XMLParser.fromString(externalWorkItemXML);
      if (node instanceof DocumentImpl) {
        DocumentImpl document = (DocumentImpl) node;
        node = document.getElementsByTagName(XML_MAIN).item(0);
      }
      if (node instanceof Element)
        this.dataFromElement( (Element) node);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }


  public void fromString(String externalWorkItemXML) {
    try {
      Node node = XMLParser.fromString(externalWorkItemXML);
      if (node instanceof DocumentImpl) {
        DocumentImpl document = (DocumentImpl) node;
        node = document.getElementsByTagName(XML_MAIN).item(0);
      }
      if (node instanceof Element)
        this.fromElement( (Element) node);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }


  /**
   *
   * @return String
   */
  public String toXML() {

    String str =  XMLParser.toString(this.toElement());
    return str;
  }
  /**
   *
   * @return String
   */
  public String toTCPIP(){
    return this.toXML() + '\n';
  }
  /**
   *
   * @param document Document
   * @return Element
   */
  private Element dataToElement(Document document) {
    Element result = document.createElement(DATA);
    for (int i = 0; i < _data.size(); i++) {
      CaseDataElement data = (CaseDataElement) _data.get(i);
      Element dataElement = data.toElement(document);
      result.appendChild(dataElement);
    }
    return result;
  }

  /**
   *
   * @param element Element
   */
  private void dataFromElement(Element element) {
    NodeList nl = XMLParser.getFirstElement(element, DATA).getChildNodes();
    for (int i = 0; i < nl.getLength(); i++) {
      Node node = nl.item(i);
      if (node instanceof Element) {
        CaseDataElement data = new CaseDataElement( (Element) node);
        _data.add(data);
      }
    }

  }

  public void setFileName(String fileName) {
    this._fileName = fileName;
  }

  public String getFileName() {
    return _fileName;
  }


  public void setFilePath(String filePath) {
    this._filePath = filePath;
  }

  public String getFilePath() {
    return _filePath;
  }


  /**
   *
   * @return String
   */
  public String getCaseID() {
    return _caseID;
  }

  /**
   *
   * @return String
   */
  public void setCaseID(String caseID) {
    _caseID = caseID;
  }

  /**
   *
   * @return String
   */
  public void setDecomopositionID(String decompositionID) {
    _decompositionID = decompositionID;
  }

  /**
   *
   * @return String
   */
  public String getDecomopositionID() {
    return _decompositionID;
  }

  /**
   *
   * @return String
   */
  public String getStecificationID() {
    return _specificationID;
  }

  /**
   *
   * @return String
   */
  public void setSpecificationID(String specificationID) {
    _specificationID = specificationID;
  }

  /**
   *
   * @param i int
   * @return CaseDataElement
   */
  public CaseDataElement getDataElement( int i) {
    if (_data.size() > i) {
      return (CaseDataElement) _data.get(i);
    }
    return null;
  }
  /**
   *
   * @return int
   */
  public int dataSize() {
    return _data.size();
  }
  /**
   *
   * @param d Document
   * @param file String
   */
  public void save(Document d, String file) {
    XMLParser.toFile(d, file);
  }
  /**
   *
   * @param e Element
   * @return String
   */
  public String save(Element e) {
    return XMLParser.toString(e);
  }
}
