package yawlservice;

import org.w3c.dom.*;

public class ExternalDataElement {

  private static final String VARIABLE = "variable";
  private static final String TYPE = "type";
  private static final String VALUE = "value";
  private static final String NAME = "name";
  private static final String INPUT = "input";
  private static final String OUTPUT = "output";

  private String _type;
  private String _value;
  private String _name;
  private boolean _output = false;
  private boolean _input = false;

  //constructors
  public ExternalDataElement(String p_name, String p_type, String p_value) {
    super();
    this._type = validate(p_type);
    this._name = validate(p_name);
    this.setValue(p_value);
  }

  public ExternalDataElement(Element element) {
    super();
    this.fromElement(element);
  }

  public String getType() {
    return _type;
  }

  public String getName() {
    return _name;
  }

  public void setValue(String p_value) {
    if (p_value == null) {
      p_value = "NOT_DEFINED";
    }
    _value = p_value;
  }

  public String getValue() {
    return _value;
  }

  public boolean isInput() {
    return _input;
  }

  public void setInput(boolean b) {
    _input = b;
  }

  public boolean isOutput() {
    return _output;
  }

  public void setOutput(boolean b) {
    _output = b;
  }

  private String validate(String attribute) {
    if (attribute == null) {
      attribute = "";
    }
    return attribute;
  }

  public Element toElement(Document document) {

    Element name = document.createElement(NAME);
    name.appendChild(document.createTextNode(validate(_name)));

    Element type = document.createElement(TYPE);
    type.appendChild(document.createTextNode(validate(_type)));

    Element value = document.createElement(VALUE);
    value.appendChild(document.createTextNode(validate(_value)));

    Element variable = document.createElement(VARIABLE);
    variable.setAttribute(INPUT, Boolean.toString(_input));
    variable.setAttribute(OUTPUT, Boolean.toString(_output));
    variable.appendChild(name);
    variable.appendChild(type);
    variable.appendChild(value);

    return variable;
  }

  public void fromElement(Element element) {
    if (element != null) {
      _name = validate(XMLParser.getSimpleElementText(element, NAME));
      _type = validate(XMLParser.getSimpleElementText(element, TYPE));
      Boolean b = new Boolean(element.getAttribute(INPUT));
      this._input = b.booleanValue();
      b = new Boolean(element.getAttribute(OUTPUT));
      this._output = b.booleanValue();
      this.setValue(XMLParser.getSimpleElementText(element, VALUE));
    }
  }

}
