package yawlservice;

import java.util.*;
import javax.xml.parsers.*;

import org.apache.xerces.dom.*;
import org.w3c.dom.*;

public class ExternalWorkItem {

  private static final String XML_MAIN = "workitem";
  private static final String TASK_ID = "task";
  private static final String CASE_ID = "case";
  private static final String DATA = "data";
  private static final String SPECIFICATION_ID = "specification";
  private static final String DECOMPOSITION_ID = "decomposition";
  private static final String ENGINE_ID = "engine";

  private String _taskID;
  private String _caseID;
  private String _specificationID;
  private String _decompositionID;
  private String _engineID;
  private List<ExternalDataElement> _data;

  public ExternalWorkItem() {
    _taskID = "";
    _caseID = "";
    _specificationID = "";
    _decompositionID = "";
    _engineID = "";
    _data = new ArrayList<ExternalDataElement>();
  }

  public ExternalWorkItem(String taskID, String caseID, String specificationID,
                          String decompositionID, String engineID) {
    _taskID = taskID;
    _caseID = caseID;
    _specificationID = specificationID;
    _decompositionID = decompositionID;
    _engineID = engineID;
    _data = new ArrayList<ExternalDataElement>();
  }

  public void addInputData(String name, String type, String value) {
    ExternalDataElement data = this.addData(name, type, value);
    if (data != null) {
      data.setValue(value);
      data.setInput(true);
    }
  }

  public ExternalDataElement addData(String name, String type, String value) {
    ExternalDataElement data = this.getData(name);
    if (data == null) {
      data = new ExternalDataElement(name, type, value);
      _data.add(data);
    }
    return data;
  }

  private ExternalDataElement getData(String name) {
    ExternalDataElement data = null;
    boolean found = false;
    Iterator<ExternalDataElement> iter = _data.iterator();
    while (iter.hasNext() && !found) {
      data = (ExternalDataElement) iter.next();
      found = data.getName().equals(name);
    }
    return found ? data : null;
  }

  public void addOutputData(String name, String type, String value) {
    ExternalDataElement data = this.addData(name, type, value);
    if (data != null) {
      data.setOutput(true);
    }
  }

  public Element toElement() {
    Element root = null;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      DOMImplementation domImpl = builder.getDOMImplementation();
      Document document = domImpl.createDocument(null, "available", null);
      root = document.createElement(XML_MAIN);

      Element taskID = document.createElement(TASK_ID);
      taskID.appendChild(document.createTextNode(_taskID));
      root.appendChild(taskID);

      Element caseID = document.createElement(CASE_ID);
      caseID.appendChild(document.createTextNode(_caseID));
      root.appendChild(caseID);

      Element decompositionID = document.createElement(DECOMPOSITION_ID);
      decompositionID.appendChild(document.createTextNode(_decompositionID));
      root.appendChild(decompositionID);

      Element engineID = document.createElement(ENGINE_ID);
      engineID.appendChild(document.createTextNode( (_engineID != null) ?
          _engineID : ""));
      root.appendChild(engineID);

      Element specID = document.createElement(SPECIFICATION_ID);
      specID.appendChild(document.createTextNode(_specificationID));
      root.appendChild(specID);

      Element data = this.dataToElement(document);
      root.appendChild(data);
    }
    catch (ParserConfigurationException e) {
    }
    return root;
   }

  public void fromElement(Element element) {
    if (element != null) {
      this._taskID = XMLParser.getSimpleElementText(element, TASK_ID);
      this._caseID = XMLParser.getSimpleElementText(element, CASE_ID);
      this._specificationID = XMLParser.getSimpleElementText(element,
          SPECIFICATION_ID);
      this._decompositionID = XMLParser.getSimpleElementText(element,
          DECOMPOSITION_ID);
      this._engineID = XMLParser.getSimpleElementText(element, ENGINE_ID);
      this.dataFromElement(element);
    }
  }

  public void dataFromString(String externalWorkItemXML) {
    try {
      Node node = XMLParser.fromString(externalWorkItemXML);
      if (node instanceof DocumentImpl) {
        DocumentImpl document = (DocumentImpl) node;
        node = document.getElementsByTagName(XML_MAIN).item(0);
      }
      if (node instanceof Element) {
        this.dataFromElement( (Element) node);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void fromString(String externalWorkItemXML) {
    try {
      Node node = XMLParser.fromString(externalWorkItemXML);
      if (node instanceof DocumentImpl) {
        DocumentImpl document = (DocumentImpl) node;
        node = document.getElementsByTagName(XML_MAIN).item(0);
      }
      if (node instanceof Element) {
        this.fromElement( (Element) node);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public String toXML() {
    return XMLParser.toString(this.toElement());
  }

  public String toTCPIP() {
    return this.toXML() + '\n';
  }

  private Element dataToElement(Document document) {
    Element result = document.createElement(DATA);
    for (int i = 0; i < _data.size(); i++) {
      ExternalDataElement data = (ExternalDataElement) _data.get(i);
      Element dataElement = data.toElement(document);
      result.appendChild(dataElement);
    }
    return result;
  }

  private void dataFromElement(Element element) {
    Element main = XMLParser.getFirstElement(element, DATA);
    NodeList nl = main.getChildNodes();

    for (int i = 0; i < nl.getLength(); i++) {
      Node node = nl.item(i);
      if (node instanceof Element) {
        ExternalDataElement data = new ExternalDataElement( (Element) node);
        _data.add(data);
      }
    }
  }

  public String getSpecificationID() {
    return _specificationID;
  }

  public String getCaseID() {
    return _caseID;
  }

  public String getTaskID() {
    return this._taskID;
  }

  public String getEngineID() {
    return _engineID;
  }

  public void setEngineID(String engineID) {
    _engineID = engineID;
  }

  public String getDecompositionID() {
    return _decompositionID;
  }

  public int dataSize() {
    return _data.size();
  }

  public ExternalDataElement getData(int i) {
    if (_data.size() > i) {
      return (ExternalDataElement) _data.get(i);
    }
    return null;
  }

  public void save(Document d, String file) {
    XMLParser.toFile(d, file);
  }

  public String save(Element e) {
    return XMLParser.toString(e);
  }
}
